/*
 * Project name:
     IR_Grid_demo.mcpar
 * Generated by:
     Visual TFT
 * Date of creation
     8/30/2018
 * Test configuration:
     MCU:             STM32F407ZG
     Dev.Board:       Mikromedia_Plus_for_STM32_ARM
                      http://www.mikroe.com/mikromedia/plus/stm32-m4/
     Oscillator:      150000000 Hz
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/mikroc/arm/
 */


#include <stddef.h>
#include "__irgrid_driver.h"
#include "IR_Grid_demo_objects.h"
#include "color_scale.h"
#include "fps.h"

typedef unsigned int    (*T_i2c_startFp)                ();
typedef unsigned int    (*T_i2c_writeFp)                (unsigned char, unsigned char*, unsigned long, unsigned long);
typedef void            (*T_i2c_readFp)                 (unsigned char, unsigned char*, unsigned long, unsigned long);

typedef struct
{
        T_i2c_startFp       i2cStart;
        T_i2c_writeFp       i2cWrite;
        T_i2c_readFp        i2cRead;

}T_i2c_obj;

const T_i2c_obj _MIKROBUS1_I2C =
{
    I2C2_Start,
    I2C2_Write,
    I2C2_Read
};

#define GET_PIX_NOR(col,row)   		ir_tmp[((col) * 4) + (row)]
#define GET_PIX_VIR(col,row)	    ir_vir[((col) * 7) + (row)]
#define SET_PIX_VIR(val,col,row)	ir_vir[((col) * 7) + (row)] = (val)

uint8_t frame_type;
uint8_t BOXS;
uint8_t V_OFF;
uint8_t H_OFF;

float   ir_tmp[ 64 ];
float   ir_vir[ 217 ];

void main() 
{
  I2C2_Init_Advanced( 400000, &_GPIO_MODULE_I2C2_PF01);
  Start_TP();
  irgrid_i2cDriverInit( (const uint8_t*)NULL, (const uint8_t*)&_MIKROBUS1_I2C, 0x50, 0x60);
  Delay_ms( 100 );
  irgrid_deviceInit( 16 );

  TFT_Set_Brush( 1,CL_WHITE, 0, 0, 0, 0 );
  TFT_Set_Font( TFT_defaultFont, CL_BLACK, FO_HORIZONTAL );
  TFT_Fill_Screen( CL_WHITE );
  color_scale_init();
  fps_init();
  
  while (1) 
  {
  
    uint8_t rc, cc;
    uint8_t i;
    uint8_t txt[ 64 ];

    //Check_TP();


  frame_type = 0;


    if( !irgrid_measure() )
    {
        irgrid_get_temperature( &ir_tmp[ 0 ] );
    }


	switch (frame_type)
	{
		case 0 :
		{
			BOXS   = 18;		// SQUARE PIXEL SIZE
			V_OFF  = 80;		// VERTICAL OFFSET
			H_OFF  = 16;		// HORTIZONTAL

    		for (rc = 0; rc < 4; rc++)
    		{
    		    for (cc = 0; cc < 16; cc++)
    		    {
    		        uint16_t color = color_scale_rgb_conversion( GET_PIX_NOR(cc, rc) );

    		        TFT_Set_Pen( color , 1 );
    		        TFT_Set_Brush( 1, color, 0, 0, 0, 0 );
    		        TFT_Rectangle( H_OFF + cc * BOXS, V_OFF + rc * BOXS, H_OFF + ((cc + 1) * BOXS) - 1, V_OFF + ((rc + 1) * BOXS) - 1 );
    		    }
    		}
		}
    	case 1 :
    	{
			BOXS  = 14;			// SQUARE PIXEL SIZE
			V_OFF = 80;			// VERTICAL OFFSET
			H_OFF = 20;			// HORTIZONTAL

			for (rc = 0; rc <= 6; rc++)
			{
				for (cc = 0; cc <= 30; cc++)
				{
					if (((rc % 2) == 0) && ((cc % 2) == 0))
					{
						float tmp;

						tmp = GET_PIX_NOR((cc / 2), (rc / 2));
						SET_PIX_VIR(tmp, cc, rc);
					}

					if (((rc % 2) == 0) && ((cc % 2) != 0))
					{
						float tmp1;
						float tmp2;

						tmp1 = GET_PIX_NOR((cc / 2), (rc / 2));
						tmp2 = GET_PIX_NOR((cc / 2) + 1, (rc / 2));

						tmp1 = (tmp1 + tmp2) / 2;
						SET_PIX_VIR(tmp1, cc, rc);
					}

					if (((rc % 2) != 0) && ((cc % 2) == 0))
					{
						float tmp1;
						float tmp2;

						tmp1 = GET_PIX_NOR((cc / 2), (rc / 2));
						tmp2 = GET_PIX_NOR((cc / 2), (rc / 2) + 1);

						tmp1 = (tmp1 + tmp2) / 2;
						SET_PIX_VIR(tmp1, cc, rc);
					}

					if (((rc % 2) != 0) && ((cc % 2) != 0))
					{
						float tmp1;
						float tmp2;
						float tmp3;
						float tmp4;

						tmp1 = GET_PIX_NOR((cc / 2), (rc / 2));
						tmp2 = GET_PIX_NOR((cc / 2) + 1, (rc / 2));
						tmp3 = GET_PIX_NOR((cc / 2), (rc / 2) + 1);
						tmp4 = GET_PIX_NOR((cc / 2) + 1, (rc / 2) + 1);

						tmp1 = (tmp1 + tmp2 + tmp3 + tmp4) / 4;
						SET_PIX_VIR(tmp1, cc, rc);
					}
				}
			}

			for (rc = 0; rc < 7; rc++)
			{
			    for (cc = 0; cc < 31; cc++)
			    {
			        uint16_t color = color_scale_rgb_conversion( GET_PIX_VIR(cc, rc) );

			        TFT_Set_Pen( color , 1 );
			        TFT_Set_Brush( 1, color, 0, 0, 0, 0 );
			        TFT_Rectangle( H_OFF + cc * BOXS, V_OFF + rc * BOXS, H_OFF + ((cc + 1) * BOXS) - 1, V_OFF + ((rc + 1) * BOXS) - 1 );
			    }
			}
		}
	}

//    fps_rate = (100.0 / (float)(fps_c - last_c));
//    last_c = fps_c;
//    FloatToStr( fps_rate, txt );
//    txt[ 4 ] = 0;
//
//    TFT_Set_Pen( 0xFFFF , 1 );
//    TFT_Set_Brush( 1, 0xFFFF, 0, 0, 0, 0 );
//    TFT_Rectangle( 10, 10, 100, 20 );
//    TFT_Write_Text( Ltrim(txt), 10, 10 );
  }
}